@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # PyHoca CLI is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -27,15 +27,11 @@ import os
 import copy
 import time
 import getpass
-from types import *
 from gevent import socket
 
 # for debugging
 import pprint
 
-# a list of available X2Go print actions
-from x2go.defaults import X2GO_PRINT_ACTIONS
-
 # use current_home as user home dir
 current_home = os.path.expanduser("~")
 
@@ -330,10 +326,28 @@ class PyHocaCLI(x2go.X2GoClient):
             self._pyhoca_logger = copy.deepcopy(logger)
             self._pyhoca_logger.tag = 'PyHocaCLI'
 
+        _backend_kwargs = {}
+        if self.args.backend_controlsession is not None:
+            _backend_kwargs['control_backend'] = self.args.backend_controlsession
+        if self.args.backend_terminalsession is not None:
+            _backend_kwargs['terminal_backend'] = self.args.backend_terminalsession
+        if self.args.backend_serversessioninfo is not None:
+            _backend_kwargs['info_backend'] = self.args.backend_serversessioninfo
+        if self.args.backend_serversessionlist is not None:
+            _backend_kwargs['list_backend'] = self.args.backend_serversessionlist
+        if self.args.backend_proxy is not None:
+            _backend_kwargs['proxy_backend'] = self.args.backend_proxy
+        if self.args.backend_sessionprofiles is not None:
+            _backend_kwargs['profiles_backend'] = self.args.backend_sessionprofiles
+        if self.args.backend_clientsettings is not None:
+            _backend_kwargs['settings_backend'] = self.args.backend_clientsettings
+        if self.args.backend_clientprinting is not None:
+            _backend_kwargs['printing_backend'] = self.args.backend_clientprinting
+
         # initialize the X2GoClient context and start the connection to the X2Go server
         self._pyhoca_logger('preparing requested X2Go session', loglevel=x2go.loglevel_NOTICE, )
 
-        x2go.X2GoClient.__init__(self, logger=liblogger)
+        x2go.X2GoClient.__init__(self, broker_url=self.args.broker_url, broker_password=self.args.broker_password, logger=liblogger, **_backend_kwargs)
 
         _profiles = self._X2GoClient__get_profiles()
         if self.args.session_profile and not _profiles.has_profile(self.args.session_profile):
@@ -341,7 +355,36 @@ class PyHocaCLI(x2go.X2GoClient):
 
         self.auth_attempts = int(self.args.auth_attempts)
 
-        if self.args.session_profile:
+
+        if args.list_profiles:
+
+            _session_profiles = self._X2GoClient__get_profiles()
+            # retrieve a session list
+            print
+            print "Available X2Go session profiles"
+            print "==============================="
+            if hasattr(_session_profiles, 'config_files') and _session_profiles.config_files is not None:
+                print "configuration files: %s" % _session_profiles.config_files
+            if hasattr(_session_profiles, 'user_config_file') and _session_profiles.user_config_file is not None:
+                print "user configuration file: %s" % _session_profiles.user_config_file
+            if hasattr(_session_profiles, 'broker_url') and _session_profiles.broker_url is not None:
+                print "X2Go Session Broker URL: %s" % _session_profiles.broker_url
+            print
+            for _profile_id in _session_profiles.profile_ids:
+                _profile_config = _session_profiles.get_profile_config(_profile_id)
+                _session_params = _session_profiles.to_session_params(_profile_id)
+                print 'Profile ID: %s' % _profile_id
+                print 'Profile Name: %s' % _session_params['profile_name']
+                print 'Profile Configuration:'
+                pprint.pprint(_profile_config)
+                print 'Derived session parameters:'
+                pprint.pprint(_session_params)
+                print
+
+            # done
+            sys.exit(0)
+
+        elif self.args.session_profile:
 
             _cmdlineopt_to_sessionopt = {
                 'command': 'cmd',
@@ -400,8 +443,12 @@ class PyHocaCLI(x2go.X2GoClient):
         connected = False
         force_password_auth = False
         _username = self.args.username or self._X2GoClient__get_session_username(self.x2go_session_hash)
-        try:
 
+        # if we still don't have a valid user name, try the broker...
+        if not _username and hasattr(self.session_profiles, 'get_broker_username'):
+            _username = self.session_profiles.get_broker_username()
+
+        try:
             _auth_count = self.auth_attempts +1
             while not connected and _auth_count:
                 try:
@@ -512,7 +559,6 @@ class PyHocaCLI(x2go.X2GoClient):
                 try:
 
                     session_duration = 0
-                    mounted = False
 
                     while self._X2GoClient__session_ok(self.x2go_session_hash):
                         time.sleep(2)
